//DESCRIPTION: Fit Width of Table to Width of First Column

  /*
  	©Copyright 2006 PDS Associates
  	
  	This script must not be distributed without first obtaining written permission of: 
  	
  	Dave Saunders, proprietor
  	PDS Associates
  	P.O. Box 127
  	Allenhurst, NJ 07711-0127
  	
  	e-mail: davesaunders@pdsassoc.com
   	
  	Version 1.00 (May 17, 2006)
  */

#target indesign

// Method returns max of array of numbers
Array.prototype.maxOf = function() { 
	var result = Number.NEGATIVE_INFINITY;
	for (var j = 0; this.length > j; j++) {
		result = Math.max(result, this[j]);
	}
	return result
} 

if ((app.documents.length == 0) || (app.selection.length == 0)) { exit() }

setUnits();
aDoc = app.activeDocument;
aSel = app.selection[0];
//aTable = findTable(aSel);
allTables = findTable(aSel);
if (!(allTables instanceof Table)){
for (var i = 0; i<allTables.length; i++){
var aTable = allTables[i];
main(aTable);
}
} else {main(allTables)}
resetUnits();

function main(t){
tWidth = getTargetWidth(t);
n = t.columnCount;
w = t.width;
for (j = n-1; j >= 0; j--) {
t.columns[j].width = t.columns[j].width * (tWidth)/w;
}
}
function getTargetWidth(table) {
	var aStory = table.parent.parentStory;
	var tf = table.parent;
	var colWidth = getWidthOfColumn(tf);
	var aChar = aStory.characters[table.storyOffset - 1];
	colWidth = colWidth - aChar.firstLineIndent - aChar.leftIndent - aChar.rightIndent;
	// adjust for stroke widths of cells in first text frame
	var Lmax = table.rows.everyItem().leftEdgeStrokeWeight.maxOf();
	var Rmax = table.rows.everyItem().rightEdgeStrokeWeight.maxOf();
	colWidth = colWidth - (Lmax + Rmax)/2;
	return colWidth * tf.horizontalScale/100;
}

function getWidthOfColumn(tf) {
	var rawWidth =  tf.geometricBounds[3] - tf.geometricBounds[1];
	// Adjust for text frame inset and stroke weight
	var strokeMultiplier = strokeAlign(tf);
	var effectiveStrokeWeight = tf.strokeWeight * strokeMultiplier;
	var insets = tf.textFramePreferences.insetSpacing;
	if (insets.constructor.name == "Number") {
		insets = [insets, insets, insets, insets];
	} 
	rawWidth = (rawWidth - effectiveStrokeWeight - insets[1] - insets[3]);
	// Adjust for multicolumn frame
	with (tf.textFramePreferences) {
		var rawWidth = (rawWidth - (textColumnGutter * (textColumnCount - 1)))/textColumnCount;
	}
	return rawWidth;
}

function setUnits() {
	//save users measurement preferences (userHoriz and userVert are global for reset)
	userHoriz = app.documents[0].viewPreferences.horizontalMeasurementUnits;
	userVert =  app.documents[0].viewPreferences.verticalMeasurementUnits;
	app.documents[0].viewPreferences.horizontalMeasurementUnits = MeasurementUnits.points;
	app.documents[0].viewPreferences.verticalMeasurementUnits = MeasurementUnits.points;
}

function resetUnits() {
	app.documents[0].viewPreferences.horizontalMeasurementUnits = userHoriz;
	app.documents[0].viewPreferences.verticalMeasurementUnits = userVert;
}

function strokeAlign(tf) {
	var Aligns = [StrokeAlignment.outsideAlignment, StrokeAlignment.centerAlignment, StrokeAlignment.insideAlignment];
	for (var j = 0; Aligns.length > j; j++) {
		if (tf.strokeAlignment == Aligns[j]) return j
	}
	errorExit("TextFrame has unrecognized stroke alignment.");
}

function findTable(obj) {
	if ((obj.constructor.name == "TextFrame") && (obj.parentStory.tables.length >0)) {return obj.parentStory.tables}
	if (obj.constructor.name == "Table") { return obj }
	while (obj.constructor.name != "Table") {
		obj = obj.parent;
		if (obj.constructor.name == "Application") {
			errorExit("Can't locate table");
		}
	}
	return obj;
}

function errorExit(message) {
	if (arguments.length > 0) {
		if (app.version != 3) { beep() } // CS2 includes beep() function.
		alert(message);
	}
	resetUnits();
	exit(); // CS exits with a beep; CS2 exits silently.
}
